
----------------------------------------------------------------
-- [v1.01.00] Localized Functions and Tables to speed things up
----------------------------------------------------------------
local _L = nUI_InfoPanel_GuildInfo_L;
local uiScaling = GetCVar("useUiScale");
local uiScale = tonumber(GetCVar("uiScale"));
local updateTimer       = math.floor(GetTime());
local path = "Interface\\AddOns\\nUI_InfoPanel_GuildInfo\\";

---------------------------------------------------
-- [v1.01.00] Initialise Variables
---------------------------------------------------
-- Data that is adjustable by the user
GuildInfo_Defaults = {};
GuildInfo_Defaults.UpdateInterval = 30;
if ( uiScaling == 0 or uiScale == 1 ) then
	GuildInfo_Defaults.FontSize = 10;
else
	GuildInfo_Defaults.FontSize = 14;
end
GuildInfo_Defaults.CustomMessage = _L["CUSTOM_MESSAGE"];
GuildInfo_Defaults.CustomImage = _L["CUSTOM_IMAGE"];

-- Data that is fixed and used by default if no CustomSettings are used
GuildInfo = {};
GuildInfo.Frame = nil;
GuildInfo.LastUpdate = 0;
GuildInfo.Message = "";

local Backdrop = {                         
	bgFile = "Interface/Tooltips/UI-Tooltip-Background", 
        	edgeFile = "Interface/Tooltips/UI-Tooltip-Border", 
        	tile = true, 
        	tileSize = 16, 
        	edgeSize = 16, 
        	insets = { left = 4, right = 4, top = 4, bottom = 4 }
        	};                       

local Backcolor = { 
        	red=0,green=0,blue=0,alpha=1 
        	};

local Bordercolor = { 
        	red=255,green=255,blue=0,alpha=1 
        	}; 


--------------------------------------------------------------
-- [v1.01.00] Get Guild Info
--------------------------------------------------------------
local function GuildInfo_GetGuildInfo()
	local guildName = GetGuildInfo("player");
	return guildName or nil;
end

---------------------------------------------------
-- [v1.01.00] Update the Frame with current information
---------------------------------------------------
local function GuildInfo_UpdateFrame(ForceUpdate)

	-- We may want to force the update so only test interval if we are using OnUpdate function
	local TimeNow = GetTime();
	if ( not ForceUpdate ) then
		if TimeNow < ( GuildInfo.LastUpdate + GuildInfo_Defaults.UpdateInterval ) then return; 	end
	end
	GuildInfo.LastUpdate = TimeNow;

	local Path, Size, Flags;

	local imagefile = nil;
	if ( GuildInfo_Defaults.CustomImage ) then
		imagefile = path .. GuildInfo_Defaults.CustomImage;
		GuildInfo.Frame.Image.Texture:SetTexture(imagefile,false);
	else
		GuildInfo.Frame.Image.Texture:SetTexture(imagefile,false);
		GuildInfo.Frame.Image.Texture:SetTexture(Backcolor.red,Backcolor.green,Backcolor.blue, Backcolor.alpha);
	end

	GuildInfo.Message = GetGuildRosterMOTD();

	Path, Size, Flags = GuildInfo.Frame.Message.Text:GetFont()
	GuildInfo.Frame.Message.Text:SetFont(Path,GuildInfo_Defaults.FontSize,Flags);
	GuildInfo.Frame.Message.Text:SetText(GuildInfo.Message);  

	Path, Size, Flags = GuildInfo.Frame.CustomMessage.Text:GetFont()
	GuildInfo.Frame.CustomMessage.Text:SetFont(Path,GuildInfo_Defaults.FontSize,Flags);
	GuildInfo.Frame.CustomMessage.Text:SetText(GuildInfo_Defaults.CustomMessage);  
end

---------------------------------------------------
-- [v1.01.00] Initialise the whole Frame
---------------------------------------------------
local function GuildInfo_InitFrame()

	local Path, Size, Flags;
	
	-- Create the Frame itself
	local frame = CreateFrame("Frame","GuildInfo_Frame",UIParent);
    	frame:SetWidth(250);        
    	frame:SetHeight(100);
   	frame:SetAlpha(1.0);        
   	frame:SetClampedToScreen(true);
	frame:SetPoint("CENTER",UIParent,"CENTER",0,0);
	frame:SetBackdrop(Backdrop);
	frame:SetBackdropColor(Backcolor.red,Backcolor.green,Backcolor.blue, Backcolor.alpha);
	frame:SetBackdropBorderColor(Bordercolor.red, Bordercolor.green,Bordercolor.blue, Bordercolor.alpha);

	frame.Image  = CreateFrame("Frame","GuildInfo_Image",frame);
	frame.Image:SetWidth(128);
	frame.Image:SetHeight(128);
	frame.Image:SetAlpha(1.0);
	frame.Image:SetBackdrop(Backdrop);
	frame.Image:SetBackdropColor(Backcolor.red,Backcolor.green,Backcolor.blue, Backcolor.alpha);
	frame.Image:SetBackdropBorderColor(Bordercolor.red, Bordercolor.green,Bordercolor.blue, Bordercolor.alpha);
	frame.Image:SetPoint("TOPLEFT",frame,"TOPLEFT",10,10);

	frame.Image.Texture = frame.Image:CreateTexture("GuildInfo_Image_Texture","OVERLAY");
	frame.Image.Texture:SetAllPoints(frame.Image);

	frame.Message = CreateFrame("Frame","GuildInfo_Message",frame);
	frame.Message:SetWidth(150);
	frame.Message:SetHeight(60);
	frame.Message:SetAlpha(1.0);
	frame.Message:SetPoint("BOTTOMLEFT",frame.Image, "BOTTOMRIGHT",10,0);
	frame.Message:SetPoint("TOPRIGHT",frame,"TOPRIGHT",-10,10);
	frame.Message:SetBackdrop(Backdrop);
	frame.Message:SetBackdropColor(Backcolor.red,Backcolor.green,Backcolor.blue, Backcolor.alpha);
	frame.Message:SetBackdropBorderColor(Bordercolor.red, Bordercolor.green,Bordercolor.blue, Bordercolor.alpha);

	frame.Message.Text = frame.Message:CreateFontString("GuildInfo_Message_Text","ARTWORK","GameFontNormal");
	Path, Size, Flags = frame.Message.Text:GetFont()
	frame.Message.Text:SetFont(Path,GuildInfo_Defaults.FontSize,Flags);
	frame.Message.Text:SetAllPoints(frame.Message);
	frame.Message.Text:SetJustifyH( "CENTER" );
	frame.Message.Text:SetJustifyV( "MIDDLE" );
	frame.Message.Text:SetTextColor( 1, 1, 0, 1 );
	frame.Message.Text:SetText("");

	frame.CustomMessage = CreateFrame("Frame","GuildInfo_CustomMessage",frame);
	frame.CustomMessage:SetWidth(150);
	frame.CustomMessage:SetHeight(60);
	frame.CustomMessage:SetAlpha(1.0);
	frame.CustomMessage:SetPoint("LEFT",frame,"LEFT",10,0);
	frame.CustomMessage:SetPoint("BOTTOMRIGHT",frame,"BOTTOMRIGHT",-10,-10);
	frame.CustomMessage.Text = frame.CustomMessage:CreateFontString("GuildInfo_CustomMessage_Text","ARTWORK","GameFontNormal");
	frame.CustomMessage:SetBackdrop(Backdrop);
	frame.CustomMessage:SetBackdropColor(Backcolor.red,Backcolor.green,Backcolor.blue, Backcolor.alpha);
	frame.CustomMessage:SetBackdropBorderColor(Bordercolor.red, Bordercolor.green,Bordercolor.blue, Bordercolor.alpha);

	Path, Size, Flags = frame.CustomMessage.Text:GetFont()
	frame.CustomMessage.Text:SetFont(Path,GuildInfo_Defaults.FontSize,Flags);
	frame.CustomMessage.Text:SetAllPoints(frame.CustomMessage);
	frame.CustomMessage.Text:SetJustifyH( "CENTER" );
	frame.CustomMessage.Text:SetJustifyV( "MIDDLE" );
	frame.CustomMessage.Text:SetTextColor( 1, 1, 0, 1 );
	frame.CustomMessage.Text:SetText("");
    
	-- Assign the OnUpdate Routine to keep the Frame updated every second
	frame:SetScript("OnUpdate",GuildInfo_UpdateFrame);
	
	-- Return this frame for use by the addon
	return frame;
end

-----------------------------------------------------------------------
-- [v1.01.00] OnEvent function for this addon
-----------------------------------------------------------------------
local function GuildInfo_OnEvent()
	if event == "PLAYER_ENTERING_WORLD" then
		GuildInfo_UpdateFrame(true);
	elseif event == "GUILD_MOTD" then
		GuildInfo_UpdateFrame(true);
	elseif (event == "VARIABLES_LOADED") then
		GuildRoster();
	elseif (event == "GUILD_INVITE_REQUEST") then
		GuildRoster();
	elseif (event == "GUILD_INVITE_CANCEL") then
		GuildRoster();
	elseif (event == "PLAYER_GUILD_UPDATE") then
		
		if(math.floor(GetTime()) > updateTimer ) then
			if ( GuildInfo_GetGuildInfo() ) then
				GuildInfo_UpdateFrame(true);	
			end		
			updateTimer = math.floor(GetTime());         	        
		end
	elseif (event == "GUILD_ROSTER_UPDATE") then
		if(math.floor(GetTime()) > updateTimer ) then
			if ( GuildInfo_GetGuildInfo() ) then
				GuildInfo_UpdateFrame(true);	
			end		
			updateTimer = math.floor(GetTime());         	        
		end
	end
end

-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
-- [v1.01.00] Check Slash Commands
-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
local function GuildInfo_SlashCommands(msg)
		local OrigMsg = msg;
        		msg = string.lower(msg);
		local args = {};
		for word in string.gmatch(msg,"[^%s]+") do
			table.insert(args,word);
		end
		DefaultsChanged = false;
		if ( args[1] == "autoupdate" ) then
			if ( args[2] and tonumber(args[2]) >= 0 and tonumber(args[2]) <= 60) then
				GuildInfo_Defaults.UpdateInterval = tonumber(args[2]);
			else
				GuildInfo_Defaults.UpdateInterval = 30;
			end
			DefaultsChanged = true;
		elseif ( args[1] == "fontsize" ) then
			if ( args[2] and tonumber(args[2]) >= 8 and tonumber(args[2]) <= 20) then
				GuildInfo_Defaults.FontSize = tonumber(args[2]);
			else
				if ( uiScaling == 0 or uiScale == 1 ) then
					GuildInfo_Defaults.FontSize = 10;
				else
					GuildInfo_Defaults.FontSize = 14;
				end
			end
			DefaultsChanged = true;
		elseif ( args[1] == "image" ) then
			local command, rest = msg:match("^(%S*)%s*(.-)$");
			if ( rest ) then
				GuildInfo_Defaults.CustomImage = rest;
			else
				GuildInfo_Defaults.CustomImage = nil;
				ReloadUI();
			end
			DefaultsChanged = true;
		elseif ( args[1] == "message" ) then
			local command, rest = OrigMsg:match("^(%S*)%s*(.-)$");
			if ( rest ) then
				GuildInfo_Defaults.CustomMessage = rest;
			else
				GuildInfo_Defaults.CustomMessage = "";
			end
			DefaultsChanged = true;
		else
			DEFAULT_CHAT_FRAME:AddMessage(_L["SLASH_COMMAND_LIST"] .. "( /guildinfo , /gi)" );
			DEFAULT_CHAT_FRAME:AddMessage(_L["SLASH_RESET"]);
			DEFAULT_CHAT_FRAME:AddMessage("autoupdate x ( x >= 0   , x <= 60 , 30  ) - " .. _L["UPDATE_SLASH"] );
			DEFAULT_CHAT_FRAME:AddMessage("fontsize x   ( x >= 8   , x <= 20 , 10 / 14  ) - " .. _L["FONT_SLASH"] );
			DEFAULT_CHAT_FRAME:AddMessage("image s ) - " .. _L["IMAGE_SLASH"]);
			DEFAULT_CHAT_FRAME:AddMessage("message s ) - " .. _L["MESSAGE_SLASH"]);
		end
		if ( DefaultsChanged ) then GuildInfo_UpdateFrame(true); end
end
-----------------------------------------------------------------------------


-----------------------------------------------------------------------------
-- [v1.01.00] Set Up Slash Commands
-----------------------------------------------------------------------------
local function GuildInfo_SetUpSlashCommands()
	SLASH_GuildInfoCmd1 = '/guildinfo';
	SLASH_GuildInfoCmd2 = '/gi';
	SlashCmdList['GuildInfoCmd'] = GuildInfo_SlashCommands;
end
-----------------------------------------------------------------------------

-----------------------------------------------------------------------
-- [v1.01.00] Create Frame Function called from outside to use this addon
-----------------------------------------------------------------------
function GuildInfo_CreateFrame()
	GuildInfo.Frame = nil;
	GuildInfo.Frame = GuildInfo_InitFrame();
	GuildInfo.Frame:SetScript( "OnEvent", GuildInfo_OnEvent );
	GuildInfo.Frame:RegisterEvent( "PLAYER_ENTERING_WORLD" );
	GuildInfo.Frame:RegisterEvent( "GUILD_MOTD");
	GuildInfo.Frame:RegisterEvent("VARIABLES_LOADED");
	GuildInfo.Frame:RegisterEvent( "GUILD_INVITE_REQUEST" );
	GuildInfo.Frame:RegisterEvent( "GUILD_INVITE_CANCEL" );
	GuildInfo.Frame:RegisterEvent( "PLAYER_GUILD_UPDATE" );
	GuildInfo.Frame:RegisterEvent( "GUILD_ROSTER_UPDATE" );
	if ( GuildInfo.Frame ) then
		GuildInfo_SetUpSlashCommands();
		GuildRoster();
	end
end


